@@ -12,9 +12,6 @@
 #include "pythoncapi_compat.h"
 #include "bitarray.h"
 
-#define IS_LE(a)  ((a)->endian == ENDIAN_LITTLE)
-#define IS_BE(a)  ((a)->endian == ENDIAN_BIG)
-
 /* set using the Python module function _set_bato() */
 static PyObject *bitarray_type_obj = NULL;
 
@@ -182,15 +179,15 @@ find_last(bitarrayobject *self, int vi, Py_ssize_t a, Py_ssize_t b)
         const Py_ssize_t byte_b = b / 8;
         const char c = vi ? 0 : ~0;
 
-        if ((res = find_last(self, vi, BITS(byte_b), b)) >= 0)
+        if ((res = find_last(self, vi, 8 * byte_b, b)) >= 0)
             return res;
 
         for (i = byte_b - 1; i >= byte_a; i--) {  /* skip bytes */
             assert_byte_in_range(self, i);
             if (c ^ self->ob_item[i])
-                return find_last(self, vi, BITS(i), BITS(i) + 8);
+                return find_last(self, vi, 8 * i, 8 * i + 8);
         }
-        return find_last(self, vi, a, BITS(byte_a));
+        return find_last(self, vi, a, 8 * byte_a);
     }
     assert(n <= 8);
     for (i = b - 1; i >= a; i--) {
@@ -215,8 +212,8 @@ r_index(PyObject *module, PyObject *args)
         return NULL;
 
 #define aa  ((bitarrayobject *) a)
-    normalize_index(aa->nbits, &start);
-    normalize_index(aa->nbits, &stop);
+    normalize_index(aa->nbits, 1, &start);
+    normalize_index(aa->nbits, 1, &stop);
     res = find_last(aa, vi, start, stop);
 #undef aa
     if (res < 0)
@@ -341,8 +338,8 @@ binary_function(PyObject *args, enum kernel_type kern, const char *format)
         }
         Py_RETURN_TRUE;
 
-    default:  /* cannot happen */
-        return NULL;
+    default:
+        Py_UNREACHABLE();
     }
 #undef aa
 #undef bb
@@ -527,7 +524,7 @@ static const char base64_alphabet[] =
     "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 
 static int
-digit_to_int(char c, int n)
+digit_to_int(int n, char c)
 {
     int i;
 
@@ -608,10 +605,10 @@ ba2base(PyObject *module, PyObject *args)
 
     le = IS_LE(aa);
     for (i = 0; i < strsize; i++) {
-        int j, k, x = 0;
+        int j, x = 0;
 
         for (j = 0; j < m; j++) {
-            k = le ? j : (m - j - 1);
+            const int k = le ? j : (m - j - 1);
             x |= getbit(aa, i * m + k) << j;
         }
         str[i] = alphabet[x];
@@ -658,11 +655,9 @@ base2ba(PyObject *module, PyObject *args)
         return NULL;
 
 #define aa  ((bitarrayobject *) a)
-    memset(aa->ob_item, 0x00, (size_t) Py_SIZE(a));
-
     le = IS_LE(aa);
     for (i = 0; i < strsize; i++) {
-        int j, k, d = digit_to_int(str[i], n);
+        int j, d = digit_to_int(n, str[i]);
 
         if (d < 0) {
             unsigned char c = str[i];
@@ -670,7 +665,7 @@ base2ba(PyObject *module, PyObject *args)
                                 "base %d, got '%c' (0x%02x)", n, c, c);
         }
         for (j = 0; j < m; j++) {
-            k = le ? j : (m - j - 1);
+            const int k = le ? j : (m - j - 1);
             setbit(aa, i * m + k, d & (1 << j));
         }
     }
@@ -693,7 +688,7 @@ grow_buffer(bitarrayobject *self)
     assert(self->readonly == 0);
 
     /* standard growth pattern */
-    newsize += (newsize >> 4) + (newsize < 8 ? 3 : 7);
+    newsize = (newsize + (newsize >> 4) + (newsize < 8 ? 3 : 7)) & ~(size_t) 3;
 
     self->ob_item = PyMem_Realloc(self->ob_item, newsize);
     if (self->ob_item == NULL) {
@@ -776,11 +771,10 @@ vl_decode(PyObject *module, PyObject *args)
     if (PyErr_Occurred())       /* from PyIter_Next() */
         return NULL;
 
-    if (b & 0x80) {
-        k = (i + PADBITS) / 7;
-        return PyErr_Format(PyExc_StopIteration,
-                            "no terminating byte found, bytes read: %zd", k);
-    }
+    if (b & 0x80)
+        return PyErr_Format(PyExc_StopIteration, "no terminating byte found, "
+                            "bytes read: %zd", (i + PADBITS) / 7);
+
     Py_RETURN_NONE;
 }
 
@@ -812,7 +806,7 @@ vl_encode(PyObject *module, PyObject *a)
         str[0] |= (0x08 >> i) * getbit(aa, i);
 
     for (i = 4; i < aa->nbits; i++) {
-        int k = (i - 4) % 7;
+        const int k = (i - 4) % 7;
 
         if (k == 0) {
             j++;
