@@ -4,7 +4,7 @@
 
    Author: Ilan Schnell
 */
-#define BITARRAY_VERSION  "2.4.0"
+#define BITARRAY_VERSION  "2.4.1"
 
 #ifdef STDC_HEADERS
 #include <stddef.h>
@@ -24,6 +24,11 @@
 
 /* --- definitions specific to Python --- */
 
+/* Py_UNREACHABLE was introduced in Python 3.7 */
+#ifndef Py_UNREACHABLE
+#define Py_UNREACHABLE() abort()
+#endif
+
 #if PY_MAJOR_VERSION >= 3
 #define IS_PY3K
 #define BYTES_SIZE_FMT  "y#"
@@ -58,6 +63,9 @@ typedef struct {
 #define ENDIAN_LITTLE  0
 #define ENDIAN_BIG     1
 
+#define IS_LE(self)  ((self)->endian == ENDIAN_LITTLE)
+#define IS_BE(self)  ((self)->endian == ENDIAN_BIG)
+
 /* the endianness string */
 #define ENDIAN_STR(endian)  ((endian) == ENDIAN_LITTLE ? "little" : "big")
 
@@ -81,7 +89,6 @@ static inline int
 getbit(bitarrayobject *self, Py_ssize_t i)
 {
     assert_nbits(self);
-    assert_byte_in_range(self, i >> 3);
     assert(0 <= i && i < self->nbits);
     return self->ob_item[i >> 3] & BITMASK(self, i) ? 1 : 0;
 }
@@ -92,7 +99,6 @@ setbit(bitarrayobject *self, Py_ssize_t i, int vi)
     char *cp, mask;
 
     assert_nbits(self);
-    assert_byte_in_range(self, i >> 3);
     assert(0 <= i && i < self->nbits);
     assert(self->readonly == 0);
 
@@ -104,23 +110,27 @@ setbit(bitarrayobject *self, Py_ssize_t i, int vi)
         *cp &= ~mask;
 }
 
-/* Return the (padded with zeros) last byte of the buffer.  When called with
-   a bitarray whose number of bits is a multiple of 8, return a NUL byte. */
+static const char bitmask_table[2][8] = {
+    {0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80},  /* little endian */
+    {0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01},  /* big endian */
+};
+
+/* character with n leading ones is: ones_table[endian][n] */
+static const char ones_table[2][8] = {
+    {0x00, 0x01, 0x03, 0x07, 0x0f, 0x1f, 0x3f, 0x7f},  /* little endian */
+    {0x00, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc, 0xfe},  /* big endian */
+};
+
+/* Return last byte in buffer with pad bits zeroed out.  The the number of
+   bits in the bitarray must not be a multiple of 8. */
 static inline char
 zeroed_last_byte(bitarrayobject *self)
 {
-    static const char mask_table[2][8] = {
-        {0, 0x01, 0x03, 0x07, 0x0f, 0x1f, 0x3f, 0x7f},  /* little endian */
-        {0, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc, 0xfe},  /* big endian */
-    };
-    Py_ssize_t t = Py_SIZE(self) - 1;  /* index of last byte in buffer */
-    int r = self->nbits % 8;           /* index into mask table */
+    const int r = self->nbits % 8;     /* index into mask table */
 
-    if (r == 0)
-        return 0x00;
+    assert(r % 8 > 0);
     assert_nbits(self);
-    assert_byte_in_range(self, t);
-    return mask_table[self->endian == ENDIAN_BIG][r] & self->ob_item[t];
+    return ones_table[IS_BE(self)][r] & self->ob_item[Py_SIZE(self) - 1];
 }
 
 /* Unless buffer is readonly, zero out pad bits.
@@ -128,7 +138,7 @@ zeroed_last_byte(bitarrayobject *self)
 static inline int
 setunused(bitarrayobject *self)
 {
-    int r = self->nbits % 8;
+    const int r = self->nbits % 8;
 
     if (r == 0)
         return 0;
@@ -137,11 +147,6 @@ setunused(bitarrayobject *self)
     return 8 - r;
 }
 
-static const char bitmask_table[2][8] = {
-    {0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80},  /* little endian */
-    {0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01},  /* big endian */
-};
-
 static const unsigned char bitcount_lookup[256] = {
 #define B2(n)  n, n + 1, n + 1, n + 2
 #define B4(n)  B2(n), B2(n + 1), B2(n + 1), B2(n + 2)
@@ -154,7 +159,8 @@ static const unsigned char bitcount_lookup[256] = {
 
 /* normalize index (which may be negative), such that:
     0 <= i <= length        (for step > 0)
-   -1 <= i <= length - 1    (for step < 0)                           */
+   -1 <= i <= length - 1    (for step < 0)
+*/
 static inline void
 normalize_index(Py_ssize_t length, Py_ssize_t step, Py_ssize_t *i)
 {
